module Widget : sig
  type widget
	  (* widget is an abstract type *)
  val name : widget -> string
	(* Returns the name (tk "path") of a widget *)
  exception IllegalWidgetType of string
end
module Protocol : sig
  exception TkError of string
  end

(* Support for Tk -textvariable option *)
module Textvariable : sig
open Widget
type textVariable
      (* TextVariable is an abstract type *)

val create : unit -> textVariable
      (* Allocation of a textVariable *)
val create_temporary : widget -> textVariable
      (* Allocation of a textVariable with lifetime associated to widget *)
val set : textVariable -> string -> unit
      (* Setting the val of a textVariable *)
val get : textVariable -> string
      (* Reading the val of a textVariable *)
val name : textVariable -> string
      (* Its tcl name *)

end
module Timer : sig
  type t

  val  add : int -> (unit -> unit) -> t
  val  remove : t -> unit
end
module Tk : sig
open Widget
open Textvariable
type bitmap =
   BitmapFile of string                 (* path of file *)
 | Predefined of string                 (* bitmap  name *)
type color =
     NamedColor of string
   | Black			(* tk keyword: black *)
   | White			(* tk keyword: white *)
   | Red			(* tk keyword: red *)
   | Green			(* tk keyword: green *)
   | Blue			(* tk keyword: blue *)
   | Yellow                     (* tk keyword: yellow *)
type cursor =
   XCursor of string 
 | XCursorFg of string * color
 | XCursortFgBg of string * color * color
 | CursorFileFg of string * color 
 | CursorMaskFile of string * string * color * color
type units =
    Pixels of int       (* specified as floating-point, but inconvenient *)
  | Centimeters of float
  | Inches of float
  | Millimeters of float
  | PrinterPoint of float
type scrollValue =
	ScrollPage of int		(* tk option: scroll <int> page *)
	| ScrollUnit of int		(* tk option: scroll <int> unit *)
	| MoveTo of float		(* tk option: moveto <float> *)
type xEvent =
    ButtonPress (* also Button, but we omit it *)
  | ButtonPressDetail of int
  | ButtonRelease
  | ButtonReleaseDetail of int
  | Circulate
  | ColorMap
  | Configure
  | Destroy
  | Enter
  | Expose
  | FocusIn
  | FocusOut
  | Gravity
  | KeyPress (* also Key, but we omit it *)
  | KeyPressDetail of string      (* /usr/include/X11/keysymdef.h *)
  | KeyRelease
  | KeyReleaseDetail of string
  | Leave
  | Map
  | Motion
  | Property
  | Reparent
  | Unmap
  | Visibility 
type modifier =
    Control
  | Shift
  | Lock
  | Button1
  | Button2
  | Button3
  | Button4
  | Button5
  | Double
  | Triple
  | Mod1
  | Mod2
  | Mod3
  | Mod4
  | Mod5
  | Meta
  | Alt 
type eventInfo =
  {
  mutable ev_Above : int;               (* tk: %a *)
  mutable ev_ButtonNumber : int;        (* tk: %b *)
  mutable ev_Count : int;               (* tk: %c *)
  mutable ev_Detail : string;           (* tk: %d *)
  mutable ev_Focus : bool;              (* tk: %f *)
  mutable ev_Height : int;              (* tk: %h *)
  mutable ev_KeyCode : int;             (* tk: %k *)
  mutable ev_Mode : string;             (* tk: %m *)
  mutable ev_OverrideRedirect : bool;   (* tk: %o *)
  mutable ev_Place : string;            (* tk: %p *)
  mutable ev_State : string;            (* tk: %s *)
  mutable ev_Time : int;                (* tk: %t *)
  mutable ev_Width : int;               (* tk: %w *)
  mutable ev_MouseX : int;              (* tk: %x *)
  mutable ev_MouseY : int;              (* tk: %y *)
  mutable ev_Char : string;             (* tk: %A *)
  mutable ev_BorderWidth : int;         (* tk: %B *)
  mutable ev_SendEvent : bool;          (* tk: %E *)
  mutable ev_KeySymString : string;     (* tk: %K *)
  mutable ev_KeySymInt : int;           (* tk: %N *)
  mutable ev_RootWindow : int;          (* tk: %R *)
  mutable ev_SubWindow : int;           (* tk: %S *)
  mutable ev_Type : int;                (* tk: %T *)
  mutable ev_Widget : widget;           (* tk: %W *)
  mutable ev_RootX : int;               (* tk: %X *)
  mutable ev_RootY : int                (* tk: %Y *)
  }
type eventField =
    Ev_Above
  | Ev_ButtonNumber
  | Ev_Count
  | Ev_Detail
  | Ev_Focus
  | Ev_Height
  | Ev_KeyCode
  | Ev_Mode
  | Ev_OverrideRedirect
  | Ev_Place
  | Ev_State
  | Ev_Time 
  | Ev_Width
  | Ev_MouseX
  | Ev_MouseY
  | Ev_Char
  | Ev_BorderWidth
  | Ev_SendEvent
  | Ev_KeySymString
  | Ev_KeySymInt
  | Ev_RootWindow
  | Ev_SubWindow
  | Ev_Type
  | Ev_Widget
  | Ev_RootX
  | Ev_RootY
type bindAction =
   BindSet of eventField list *  (eventInfo -> unit)
 | BindSetBreakable of eventField list *  (eventInfo -> unit)
 | BindRemove
 | BindExtend of eventField list *  (eventInfo -> unit)
type bindings =
	TagBindings of string		(* tk option: <string> *)
	| WidgetBindings of widget	(* tk option: <widget> *)
type index =
	  Number of int		(* no keyword  *)
        | ActiveElement         (* tk keyword: active *)
	| End		        (* tk keyword: end *)
        | Last			(* tk keyword: last *)
        | NoIndex		(* tk keyword: none *)
	| Insert		(* tk keyword: insert *)
	| SelFirst		(* tk keyword: sel.first *)
	| SelLast		(* tk keyword: sel.last *)
        | At of int		(* tk keyword: @n *)
        | AtXY of int * int     (* tk keyword: @x,y *)
        | AnchorPoint   	(* tk keyword: anchor *)
        | Pattern of string     (* no keyword *)
        | LineChar of int * int (* tk keyword: l.c *)
        | Mark of string        (* no keyword *)
        | TagFirst of string    (* tk keyword: tag.first *)
        | TagLast of string     (* tk keyword: tag.last *)
        | Embedded of widget	(* no keyword *)
type paletteType =
    GrayShades of int
  | RGBShades of int * int * int
type textMark = string
type textTag = string
type textModifier =
    CharOffset of int		(* tk keyword: +/- Xchars *)
  | LineOffset of int		(* tk keyword: +/- Xlines *)
  | LineStart			(* tk keyword: linestart *)
  | LineEnd			(* tk keyword: lineend *)
  | WordStart			(* tk keyword: wordstart *)
  | WordEnd 			(* tk keyword: wordend *)
type textIndex =
   TextIndex of index * textModifier list
 | TextIndexNone
type anchor =
	Center		(* tk option: center *)
	| E		(* tk option: e *)
	| N		(* tk option: n *)
	| NE		(* tk option: ne *)
	| NW		(* tk option: nw *)
	| S		(* tk option: s *)
	| SE		(* tk option: se *)
	| SW		(* tk option: sw *)
	| W		(* tk option: w *)
type imageBitmap =
	BitmapImage of string		(* tk option: <string> *)
type imagePhoto =
	PhotoImage of string		(* tk option: <string> *)
type justification =
	Justify_Center		(* tk option: center *)
	| Justify_Left		(* tk option: left *)
	| Justify_Right		(* tk option: right *)
type orientation =
	Horizontal		(* tk option: horizontal *)
	| Vertical		(* tk option: vertical *)
type relief =
	Flat		(* tk option: flat *)
	| Groove		(* tk option: groove *)
	| Raised		(* tk option: raised *)
	| Ridge		(* tk option: ridge *)
	| Sunken		(* tk option: sunken *)
type state =
	Active		(* tk option: active *)
	| Disabled		(* tk option: disabled *)
	| Normal		(* tk option: normal *)
type colorMode =
	Color		(* tk option: color *)
	| Gray		(* tk option: gray *)
	| Mono		(* tk option: mono *)
type arcStyle =
	Arc		(* tk option: arc *)
	| Chord		(* tk option: chord *)
	| PieSlice		(* tk option: pieslice *)
type tagOrId =
	Id of int		(* tk option: <int> *)
	| Tag of string		(* tk option: <string> *)
type arrowStyle =
	Arrow_Both		(* tk option: both *)
	| Arrow_First		(* tk option: first *)
	| Arrow_Last		(* tk option: last *)
	| Arrow_None		(* tk option: none *)
type capStyle =
	Cap_Butt		(* tk option: butt *)
	| Cap_Projecting		(* tk option: projecting *)
	| Cap_Round		(* tk option: round *)
type joinStyle =
	Join_Bevel		(* tk option: bevel *)
	| Join_Miter		(* tk option: miter *)
	| Join_Round		(* tk option: round *)
type visual =
	Best		(* tk option: best *)
	| BestDepth of int		(* tk option: {best <int>} *)
	| ClassVisual of string * int		(* tk option: {<string> <int>} *)
	| DefaultVisual		(* tk option: default *)
	| WidgetVisual of widget		(* tk option: <widget> *)
type colormap =
	NewColormap		(* tk option: new *)
	| WidgetColormap of widget		(* tk option: <widget> *)
type selectModeType =
	Browse		(* tk option: browse *)
	| Extended		(* tk option: extended *)
	| Multiple		(* tk option: multiple *)
	| Single		(* tk option: single *)
type fillMode =
	Fill_Both		(* tk option: both *)
	| Fill_None		(* tk option: none *)
	| Fill_X		(* tk option: x *)
	| Fill_Y		(* tk option: y *)
type side =
	Side_Bottom		(* tk option: bottom *)
	| Side_Left		(* tk option: left *)
	| Side_Right		(* tk option: right *)
	| Side_Top		(* tk option: top *)
type borderMode =
	Ignore		(* tk option: ignore *)
	| Inside		(* tk option: inside *)
	| Outside		(* tk option: outside *)
type alignType =
	Align_Baseline		(* tk option: baseline *)
	| Align_Bottom		(* tk option: bottom *)
	| Align_Center		(* tk option: center *)
	| Align_Top		(* tk option: top *)
type wrapMode =
	WrapChar		(* tk option: char *)
	| WrapNone		(* tk option: none *)
	| WrapWord		(* tk option: word *)
type tabType =
	TabCenter of units		(* tk option: <units> center *)
	| TabLeft of units		(* tk option: <units> left *)
	| TabNumeric of units		(* tk option: <units> numeric *)
	| TabRight of units		(* tk option: <units> right *)
type options =
	Accelerator of string		(* tk option: -accelerator <string> *)
	| ActiveBackground of color		(* tk option: -activebackground <color> *)
	| ActiveBorderWidth of units		(* tk option: -activeborderwidth <units> *)
	| ActiveForeground of color		(* tk option: -activeforeground <color> *)
	| ActiveRelief of relief		(* tk option: -activerelief <relief> *)
	| After of widget		(* tk option: -after <widget> *)
	| Align of alignType		(* tk option: -align <alignType> *)
	| Anchor of anchor		(* tk option: -anchor <anchor> *)
	| ArcStyle of arcStyle		(* tk option: -style <arcStyle> *)
	| ArrowShape of units * units * units		(* tk option: -arrowshape {<units> <units> <units>} *)
	| ArrowStyle of arrowStyle		(* tk option: -arrow <arrowStyle> *)
	| Aspect of int		(* tk option: -aspect <int> *)
	| Background of color		(* tk option: -background <color> *)
	| Before of widget		(* tk option: -before <widget> *)
	| BgStipple of bitmap		(* tk option: -bgstipple <bitmap> *)
	| BigIncrement of float		(* tk option: -bigincrement <float> *)
	| Bitmap of bitmap		(* tk option: -bitmap <bitmap> *)
	| BorderMode of borderMode		(* tk option: -bordermode <borderMode> *)
	| BorderWidth of units		(* tk option: -borderwidth <units> *)
	| CapStyle of capStyle		(* tk option: -capstyle <capStyle> *)
	| Class of string		(* tk option: -class <string> *)
	| CloseEnough of float		(* tk option: -closeenough <float> *)
	| Colormap of colormap		(* tk option: -colormap <colormap> *)
	| Colormode of colorMode		(* tk option: -colormode <colorMode> *)
	| Column of int		(* tk option: -column <int> *)
	| ColumnSpan of int		(* tk option: -columnspan <int> *)
	| Command of (unit -> unit)		(* tk option: -command <(unit -> unit)> *)
	| Confine of bool		(* tk option: -confine <bool> *)
	| Cursor of cursor		(* tk option: -cursor <cursor> *)
	| Data of string		(* tk option: -data <string> *)
	| Digits of int		(* tk option: -digits <int> *)
	| DisabledForeground of color		(* tk option: -disabledforeground <color> *)
	| ElementBorderWidth of units		(* tk option: -elementborderwidth <units> *)
	| Expand of bool		(* tk option: -expand <bool> *)
	| ExportSelection of bool		(* tk option: -exportselection <bool> *)
	| Extent of float		(* tk option: -extent <float> *)
	| FgStipple of bitmap		(* tk option: -fgstipple <bitmap> *)
	| File of string		(* tk option: -file <string> *)
	| Fill of fillMode		(* tk option: -fill <fillMode> *)
	| FillColor of color		(* tk option: -fill <color> *)
	| Font of string		(* tk option: -font <string> *)
	| Foreground of color		(* tk option: -foreground <color> *)
	| Format of string		(* tk option: -format <string> *)
	| From of float		(* tk option: -from <float> *)
	| Gamma of float		(* tk option: -gamma <float> *)
	| Geometry of string		(* tk option: -geometry <string> *)
	| Height of units		(* tk option: -height <units> *)
	| HighlightBackground of color		(* tk option: -highlightbackground <color> *)
	| HighlightColor of color		(* tk option: -highlightcolor <color> *)
	| HighlightThickness of units		(* tk option: -highlightthickness <units> *)
	| IPadX of units		(* tk option: -ipadx <units> *)
	| IPadY of units		(* tk option: -ipady <units> *)
	| ImageBitmap of imageBitmap		(* tk option: -image <imageBitmap> *)
	| ImagePhoto of imagePhoto		(* tk option: -image <imagePhoto> *)
	| In of widget		(* tk option: -in <widget> *)
	| IndicatorOn of bool		(* tk option: -indicatoron <bool> *)
	| InsertBackground of color		(* tk option: -insertbackground <color> *)
	| InsertBorderWidth of units		(* tk option: -insertborderwidth <units> *)
	| InsertOffTime of int		(* tk option: -insertofftime <int> *)
	| InsertOnTime of int		(* tk option: -insertontime <int> *)
	| InsertWidth of units		(* tk option: -insertwidth <units> *)
	| JoinStyle of joinStyle		(* tk option: -joinstyle <joinStyle> *)
	| Jump of bool		(* tk option: -jump <bool> *)
	| Justify of justification		(* tk option: -justify <justification> *)
	| LMargin1 of units		(* tk option: -lmargin1 <units> *)
	| LMargin2 of units		(* tk option: -lmargin2 <units> *)
	| Label of string		(* tk option: -label <string> *)
	| Length of units		(* tk option: -length <units> *)
	| Maskdata of string		(* tk option: -maskdata <string> *)
	| Maskfile of string		(* tk option: -maskfile <string> *)
	| Menu of widget		(* tk option: -menu <widget> *)
	| OffValue of string		(* tk option: -offvalue <string> *)
	| Offset of units		(* tk option: -offset <units> *)
	| OnValue of string		(* tk option: -onvalue <string> *)
	| Orient of orientation		(* tk option: -orient <orientation> *)
	| Outline of color		(* tk option: -outline <color> *)
	| OutlineStipple of bitmap		(* tk option: -outlinestipple <bitmap> *)
	| OverStrike of bool		(* tk option: -overstrike <bool> *)
	| PadX of units		(* tk option: -padx <units> *)
	| PadY of units		(* tk option: -pady <units> *)
	| PageAnchor of anchor		(* tk option: -pageanchor <anchor> *)
	| PageHeight of units		(* tk option: -pageheight <units> *)
	| PageWidth of units		(* tk option: -pagewidth <units> *)
	| PageX of units		(* tk option: -pagex <units> *)
	| PageY of units		(* tk option: -pagey <units> *)
	| Palette of paletteType		(* tk option: -palette <paletteType> *)
	| PostCommand of (unit -> unit)		(* tk option: -postcommand <(unit -> unit)> *)
	| RMargin of units		(* tk option: -rmargin <units> *)
	| RelHeight of float		(* tk option: -relheight <float> *)
	| RelWidth of float		(* tk option: -relwidth <float> *)
	| RelX of float		(* tk option: -relx <float> *)
	| RelY of float		(* tk option: -rely <float> *)
	| Relief of relief		(* tk option: -relief <relief> *)
	| RepeatDelay of int		(* tk option: -repeatdelay <int> *)
	| RepeatInterval of int		(* tk option: -repeatinterval <int> *)
	| Resolution of float		(* tk option: -resolution <float> *)
	| Rotate of bool		(* tk option: -rotate <bool> *)
	| Row of int		(* tk option: -row <int> *)
	| RowSpan of int		(* tk option: -rowspan <int> *)
	| ScaleCommand of (float -> unit)		(* tk option: -command <(float -> unit)> *)
	| Screen of string		(* tk option: -screen <string> *)
	| ScrollCommand of (scrollValue -> unit)		(* tk option: -command <(scrollValue -> unit)> *)
	| ScrollRegion of units * units * units * units		(* tk option: -scrollregion {<units> <units> <units> <units>} *)
	| SelectBackground of color		(* tk option: -selectbackground <color> *)
	| SelectBorderWidth of units		(* tk option: -selectborderwidth <units> *)
	| SelectColor of color		(* tk option: -selectcolor <color> *)
	| SelectForeground of color		(* tk option: -selectforeground <color> *)
	| SelectImageBitmap of imageBitmap		(* tk option: -selectimage <imageBitmap> *)
	| SelectImagePhoto of imagePhoto		(* tk option: -selectimage <imagePhoto> *)
	| SelectMode of selectModeType		(* tk option: -selectmode <selectModeType> *)
	| SetGrid of bool		(* tk option: -setgrid <bool> *)
	| Show of char		(* tk option: -show <char> *)
	| ShowValue of bool		(* tk option: -showvalue <bool> *)
	| Side of side		(* tk option: -side <side> *)
	| SliderLength of units		(* tk option: -sliderlength <units> *)
	| Smooth of bool		(* tk option: -smooth <bool> *)
	| Spacing1 of units		(* tk option: -spacing1 <units> *)
	| Spacing2 of units		(* tk option: -spacing2 <units> *)
	| Spacing3 of units		(* tk option: -spacing3 <units> *)
	| SplineSteps of int		(* tk option: -splinesteps <int> *)
	| Start of float		(* tk option: -start <float> *)
	| State of state		(* tk option: -state <state> *)
	| Sticky of string		(* tk option: -sticky <string> *)
	| Stipple of bitmap		(* tk option: -stipple <bitmap> *)
	| Stretch of bool		(* tk option: -stretch <bool> *)
	| Tabs of tabType list		(* tk option: -tabs {<tabType list>} *)
	| Tags of tagOrId list		(* tk option: -tags {<tagOrId list>} *)
	| TakeFocus of bool		(* tk option: -takefocus <bool> *)
	| TearOff of bool		(* tk option: -tearoff <bool> *)
	| Text of string		(* tk option: -text <string> *)
	| TextHeight of int		(* tk option: -height <int> *)
	| TextVariable of textVariable		(* tk option: -textvariable <textVariable> *)
	| TextWidth of int		(* tk option: -width <int> *)
	| TickInterval of float		(* tk option: -tickinterval <float> *)
	| To of float		(* tk option: -to <float> *)
	| TroughColor of color		(* tk option: -troughcolor <color> *)
	| Underline of bool		(* tk option: -underline <bool> *)
	| UnderlinedChar of int		(* tk option: -underline <int> *)
	| Value of string		(* tk option: -value <string> *)
	| Variable of textVariable		(* tk option: -variable <textVariable> *)
	| Visual of visual		(* tk option: -visual <visual> *)
	| Width of units		(* tk option: -width <units> *)
	| Window of widget		(* tk option: -window <widget> *)
	| Wrap of wrapMode		(* tk option: -wrap <wrapMode> *)
	| WrapLength of units		(* tk option: -wraplength <units> *)
	| X of units		(* tk option: -x <units> *)
	| XScrollCommand of (float -> float -> unit)		(* tk option: -xscrollcommand <(float -> float -> unit)> *)
	| XScrollIncrement of units		(* tk option: -xscrollincrement <units> *)
	| Y of units		(* tk option: -y <units> *)
	| YScrollCommand of (float -> float -> unit)		(* tk option: -yscrollcommand <(float -> float -> unit)> *)
	| YScrollIncrement of units		(* tk option: -yscrollincrement <units> *)
type options_constrs =
	CAccelerator
	| CActiveBackground
	| CActiveBorderWidth
	| CActiveForeground
	| CActiveRelief
	| CAfter
	| CAlign
	| CAnchor
	| CArcStyle
	| CArrowShape
	| CArrowStyle
	| CAspect
	| CBackground
	| CBefore
	| CBgStipple
	| CBigIncrement
	| CBitmap
	| CBorderMode
	| CBorderWidth
	| CCapStyle
	| CClass
	| CCloseEnough
	| CColormap
	| CColormode
	| CColumn
	| CColumnSpan
	| CCommand
	| CConfine
	| CCursor
	| CData
	| CDigits
	| CDisabledForeground
	| CElementBorderWidth
	| CExpand
	| CExportSelection
	| CExtent
	| CFgStipple
	| CFile
	| CFill
	| CFillColor
	| CFont
	| CForeground
	| CFormat
	| CFrom
	| CGamma
	| CGeometry
	| CHeight
	| CHighlightBackground
	| CHighlightColor
	| CHighlightThickness
	| CIPadX
	| CIPadY
	| CImageBitmap
	| CImagePhoto
	| CIn
	| CIndicatorOn
	| CInsertBackground
	| CInsertBorderWidth
	| CInsertOffTime
	| CInsertOnTime
	| CInsertWidth
	| CJoinStyle
	| CJump
	| CJustify
	| CLMargin1
	| CLMargin2
	| CLabel
	| CLength
	| CMaskdata
	| CMaskfile
	| CMenu
	| COffValue
	| COffset
	| COnValue
	| COrient
	| COutline
	| COutlineStipple
	| COverStrike
	| CPadX
	| CPadY
	| CPageAnchor
	| CPageHeight
	| CPageWidth
	| CPageX
	| CPageY
	| CPalette
	| CPostCommand
	| CRMargin
	| CRelHeight
	| CRelWidth
	| CRelX
	| CRelY
	| CRelief
	| CRepeatDelay
	| CRepeatInterval
	| CResolution
	| CRotate
	| CRow
	| CRowSpan
	| CScaleCommand
	| CScreen
	| CScrollCommand
	| CScrollRegion
	| CSelectBackground
	| CSelectBorderWidth
	| CSelectColor
	| CSelectForeground
	| CSelectImageBitmap
	| CSelectImagePhoto
	| CSelectMode
	| CSetGrid
	| CShow
	| CShowValue
	| CSide
	| CSliderLength
	| CSmooth
	| CSpacing1
	| CSpacing2
	| CSpacing3
	| CSplineSteps
	| CStart
	| CState
	| CSticky
	| CStipple
	| CStretch
	| CTabs
	| CTags
	| CTakeFocus
	| CTearOff
	| CText
	| CTextHeight
	| CTextVariable
	| CTextWidth
	| CTickInterval
	| CTo
	| CTroughColor
	| CUnderline
	| CUnderlinedChar
	| CValue
	| CVariable
	| CVisual
	| CWidth
	| CWindow
	| CWrap
	| CWrapLength
	| CX
	| CXScrollCommand
	| CXScrollIncrement
	| CY
	| CYScrollCommand
	| CYScrollIncrement
type searchSpec =
	Above of tagOrId		(* tk option: above <tagOrId> *)
	| All		(* tk option: all *)
	| Below of tagOrId		(* tk option: below <tagOrId> *)
	| Closest of units * units		(* tk option: closest <units> <units> *)
	| ClosestHalo of units * units * units		(* tk option: closest <units> <units> <units> *)
	| ClosestHaloStart of units * units * units * tagOrId		(* tk option: closest <units> <units> <units> <tagOrId> *)
	| Enclosed of units * units * units * units		(* tk option: enclosed <units> <units> <units> <units> *)
	| Overlapping of int * int * int * int		(* tk option: overlapping <int> <int> <int> <int> *)
	| Withtag of tagOrId		(* tk option: withtag <tagOrId> *)
type canvasItem =
	Arc_item		(* tk option: arc *)
	| Bitmap_item		(* tk option: bitmap *)
	| Image_item		(* tk option: image *)
	| Line_item		(* tk option: line *)
	| Oval_item		(* tk option: oval *)
	| Polygon_item		(* tk option: polygon *)
	| Rectangle_item		(* tk option: rectangle *)
	| Text_item		(* tk option: text *)
	| User_item of string		(* tk option: <string> *)
	| Window_item		(* tk option: window *)
type icccm =
	DisplayOf of widget		(* tk option: -displayof <widget> *)
	| ICCCMFormat of string		(* tk option: -format <string> *)
	| ICCCMType of string		(* tk option: -type <string> *)
	| LostCommand of (unit -> unit)		(* tk option: -command <(unit -> unit)> *)
	| Selection of string		(* tk option: -selection <string> *)
type grabStatus =
	GrabGlobal		(* tk option: global *)
	| GrabLocal		(* tk option: local *)
	| GrabNone		(* tk option: none *)
type gridopt =
	Minsize of units		(* tk option: -minsize <units> *)
	| Weight of float		(* tk option: -weight <float> *)
type menuItem =
	Cascade_Item		(* tk option: cascade *)
	| Checkbutton_Item		(* tk option: checkbutton *)
	| Command_Item		(* tk option: command *)
	| Radiobutton_Item		(* tk option: radiobutton *)
	| Separator_Item		(* tk option: separator *)
	| TearOff_Item		(* tk option: tearoff *)
type optionPriority =
	Interactive		(* tk option: interactive *)
	| Priority of int		(* tk option: <int> *)
	| StartupFile		(* tk option: startupFile *)
	| UserDefault		(* tk option: userDefault *)
	| WidgetDefault		(* tk option: widgetDefault *)
type photo =
	ImgFormat of string		(* tk option: -format <string> *)
	| ImgFrom of int * int * int * int		(* tk option: -from <int> <int> <int> <int> *)
	| ImgTo of int * int * int * int		(* tk option: -to <int> <int> <int> <int> *)
	| Shrink		(* tk option: -shrink *)
	| Subsample of int * int		(* tk option: -subsample <int> <int> *)
	| TopLeft of int * int		(* tk option: -to <int> <int> *)
	| Zoom of int * int		(* tk option: -zoom <int> <int> *)
type widgetElement =
	Arrow1		(* tk option: arrow1 *)
	| Arrow2		(* tk option: arrow2 *)
	| Beyond		(* tk option:  *)
	| Slider		(* tk option: slider *)
	| Trough1		(* tk option: trough1 *)
	| Trough2		(* tk option: trough2 *)
type comparison =
	EQ		(* tk option: == *)
	| GE		(* tk option: >= *)
	| GT		(* tk option: > *)
	| LE		(* tk option: <= *)
	| LT		(* tk option: < *)
	| NEQ		(* tk option: != *)
type markDirection =
	Mark_Left		(* tk option: left *)
	| Mark_Right		(* tk option: right *)
type textSearch =
	Backwards		(* tk option: -backwards *)
	| Count of textVariable		(* tk option: -count <textVariable> *)
	| Exact		(* tk option: -exact *)
	| Forwards		(* tk option: -forwards *)
	| Nocase		(* tk option: -nocase *)
	| Regexp		(* tk option: -regexp *)
type atomId =
	AtomId of int		(* tk option: <int> *)
type focusModel =
	FocusActive		(* tk option: active *)
	| FocusPassive		(* tk option: passive *)
type wmFrom =
	FromProgram		(* tk option: program *)
	| FromUser		(* tk option: user *)
 val bind: 
    widget -> (modifier list * xEvent) list -> bindAction -> unit
  val break : unit -> unit
val appname_get : unit ->string 
(* tk invocation: tk appname *)

val appname_set : string -> unit 
(* tk invocation: tk appname <string> *)

val bindtags : widget -> bindings list -> unit 
(* tk invocation: bindtags <widget> {<bindings list>} *)

val bindtags_get : widget -> bindings list 
(* tk invocation: bindtags <widget> *)

val cget : widget -> options_constrs -> string 
(* tk invocation: <widget> cget <options_constrs> *)

val cgets : widget -> string -> string 
(* tk invocation: <widget> cget <string> *)

val destroy : widget -> unit 
(* tk invocation: destroy <widget> *)

val grid : widget list -> options list -> unit 
(* tk invocation: grid <widget list> <options list> *)

val lower_window : widget -> unit 
(* tk invocation: lower <widget> *)

val lower_window_below : widget -> widget -> unit 
(* tk invocation: lower <widget> <widget> *)

val pack : widget list -> options list -> unit 
(* tk invocation: pack <widget list> <options list> *)

val place : widget -> options list -> unit 
(* tk invocation: place <widget> <options list> *)

val raise_window : widget -> unit 
(* tk invocation: raise <widget> *)

val raise_window_above : widget -> widget -> unit 
(* tk invocation: raise <widget> <widget> *)

val update : unit ->unit 
(* tk invocation: update *)

val update_idletasks : unit ->unit 
(* tk invocation: update idletasks *)

end
module Scale : sig
(* The scale widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val get : widget -> float 
(* tk invocation: <widget> get *)

val get_xy : widget -> int -> int -> float 
(* tk invocation: <widget> get <int> <int> *)

val identify : widget -> int -> int -> widgetElement 
(* tk invocation: <widget> <int> <int> *)

val set : widget -> float -> unit 
(* tk invocation: <widget> set <float> *)

end

module Radiobutton : sig
(* The radiobutton widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val deselect : widget -> unit 
(* tk invocation: <widget> deselect *)

val flash : widget -> unit 
(* tk invocation: <widget> flash *)

val invoke : widget -> unit 
(* tk invocation: <widget> invoke *)

val select : widget -> unit 
(* tk invocation: <widget> select *)

end

module Dialog : sig
(* The dialog commands  *)
open Tk
open Widget
open Textvariable
val create : 
  widget -> string -> string -> bitmap -> int -> string list -> int 
  (* [create parent title message bitmap default button_names] 
     cf. tk_dialog *)

val create_named :
  widget -> string -> string -> string -> bitmap -> int -> string list
       -> int 
  (* [create_named parent name title message bitmap default button_names] 
     cf. tk_dialog *)



end

module Selection : sig
(* The selection commands  *)
open Tk
open Widget
open Textvariable
val clear : icccm list -> unit 
(* tk invocation: selection clear <icccm list> *)

val get : icccm list -> string 
(* tk invocation: selection get <icccm list> *)


val handle_set : icccm list -> widget -> (int -> int -> unit) -> unit
(* tk invocation: selection handle <icccm list> <widget> <command> *)


val own_set : icccm list -> widget -> unit 
(* tk invocation: selection own <icccm list> <widget> *)


end

module Toplevel : sig
(* The toplevel widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

end

module Frame : sig
(* The frame widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

end

module Canvas : sig
(* The canvas widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val addtag : widget -> tagOrId -> searchSpec list -> unit 
(* tk invocation: <widget> addtag <tagOrId> <searchSpec list> *)

val bbox : widget -> tagOrId list -> int * int * int * int 
(* tk invocation: <widget> bbox <tagOrId list> *)

val canvasx : widget -> units -> float 
(* tk invocation: <widget> canvasx <units> *)

val canvasx_grid : widget -> units -> units -> float 
(* tk invocation: <widget> canvasx <units> <units> *)

val canvasy : widget -> units -> float 
(* tk invocation: <widget> canvasy <units> *)

val canvasy_grid : widget -> units -> units -> float 
(* tk invocation: <widget> canvasy <units> <units> *)

val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_arc : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_bitmap : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val configure_image : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_line : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_oval : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_polygon : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_rectangle : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_text : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val configure_window : widget -> tagOrId -> options list -> unit 
(* tk invocation: <widget> itemconfigure <tagOrId> <options list> *)

val coords_get : widget -> tagOrId -> float list 
(* tk invocation: <widget> coords <tagOrId> *)

val coords_set : widget -> tagOrId -> units list -> unit 
(* tk invocation: <widget> coords <tagOrId> <units list> *)

val create_arc : widget -> units -> units -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create arc <units> <units> <units> <units> <options list> *)

val create_bitmap : widget -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create bitmap <units> <units> <options list> *)

val create_image : widget -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create image <units> <units> <options list> *)

val create_line : widget -> units list -> options list -> tagOrId 
(* tk invocation: <widget> create line <units list> <options list> *)

val create_oval : widget -> units -> units -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create oval <units> <units> <units> <units> <options list> *)

val create_polygon : widget -> units list -> options list -> tagOrId 
(* tk invocation: <widget> create polygon <units list> <options list> *)

val create_rectangle : widget -> units -> units -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create rectangle <units> <units> <units> <units> <options list> *)

val create_text : widget -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create text <units> <units> <options list> *)

val create_window : widget -> units -> units -> options list -> tagOrId 
(* tk invocation: <widget> create window <units> <units> <options list> *)

val dchars : widget -> tagOrId -> index -> index -> unit 
(* tk invocation: <widget> dchars <tagOrId> <index> <index> *)

val delete : widget -> tagOrId list -> unit 
(* tk invocation: <widget> delete <tagOrId list> *)

val dtag : widget -> tagOrId -> tagOrId -> unit 
(* tk invocation: <widget> dtag <tagOrId> <tagOrId> *)

val find : widget -> searchSpec list -> tagOrId list 
(* tk invocation: <widget> find <searchSpec list> *)

val focus : widget -> tagOrId -> unit 
(* tk invocation: <widget> focus <tagOrId> *)

val focus_get : widget -> tagOrId 
(* tk invocation: <widget> focus *)

val focus_reset : widget -> unit 
(* tk invocation: <widget> focus  *)

val gettags : widget -> tagOrId -> tagOrId list 
(* tk invocation: <widget> gettags <tagOrId> *)

val icursor : widget -> tagOrId -> index -> unit 
(* tk invocation: <widget> icursor <tagOrId> <index> *)

val index : widget -> tagOrId -> index -> int 
(* tk invocation: <widget> index <tagOrId> <index> *)

val insert : widget -> tagOrId -> index -> string -> unit 
(* tk invocation: <widget> insert <tagOrId> <index> <string> *)

val itemconfigure_get : widget -> tagOrId -> string 
(* tk invocation: <widget> itemconfigure <tagOrId> *)

val lower_below : widget -> tagOrId -> tagOrId -> unit 
(* tk invocation: <widget> lower <tagOrId> <tagOrId> *)

val lower_bot : widget -> tagOrId -> unit 
(* tk invocation: <widget> lower <tagOrId> *)

val move : widget -> tagOrId -> units -> units -> unit 
(* tk invocation: <widget> move <tagOrId> <units> <units> *)


val raise_above : widget -> tagOrId -> tagOrId -> unit 
(* tk invocation: <widget> raise <tagOrId> <tagOrId> *)

val raise_top : widget -> tagOrId -> unit 
(* tk invocation: <widget> raise <tagOrId> *)

val scale : widget -> tagOrId -> units -> units -> float -> float -> unit 
(* tk invocation: <widget> scale <tagOrId> <units> <units> <float> <float> *)

val scan_dragto : widget -> int -> int -> unit 
(* tk invocation: <widget> scan dragto <int> <int> *)

val scan_mark : widget -> int -> int -> unit 
(* tk invocation: <widget> scan mark <int> <int> *)

val select_adjust : widget -> tagOrId -> index -> unit 
(* tk invocation: <widget> select adjust <tagOrId> <index> *)

val select_clear : widget -> unit 
(* tk invocation: <widget> select clear *)

val select_from : widget -> tagOrId -> index -> unit 
(* tk invocation: <widget> select from <tagOrId> <index> *)

val select_item : widget -> tagOrId 
(* tk invocation: <widget> select item *)

val select_to : widget -> tagOrId -> index -> unit 
(* tk invocation: <widget> select to <tagOrId> <index> *)

val typeof : widget -> tagOrId -> canvasItem 
(* tk invocation: <widget> type <tagOrId> *)

val xview : widget -> scrollValue -> unit 
(* tk invocation: <widget> xview <scrollValue> *)

val xview_get : widget -> float * float 
(* tk invocation: <widget> xview *)

val yview : widget -> scrollValue -> unit 
(* tk invocation: <widget> yview <scrollValue> *)

val yview_get : widget -> float * float 
(* tk invocation: <widget> yview *)

val bind : widget -> tagOrId -> 
                    (modifier list * xEvent) list -> bindAction -> unit 


end

module Pack : sig
(* The pack commands  *)
open Tk
open Widget
open Textvariable
val configure : widget list -> options list -> unit 
(* tk invocation: pack configure <widget list> <options list> *)

val forget : widget list -> unit 
(* tk invocation: pack forget <widget list> *)

val propagate_get : widget -> bool 
(* tk invocation: pack propagate <widget> *)

val propagate_set : widget -> bool -> unit 
(* tk invocation: pack propagate <widget> <bool> *)

val slaves : widget -> widget list 
(* tk invocation: pack slaves <widget> *)

end

module Listbox : sig
(* The listbox widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val activate : widget -> index -> unit 
(* tk invocation: <widget> activate <index> *)

val bbox : widget -> index -> int * int * int * int 
(* tk invocation: <widget> bbox <index> *)

val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val curselection : widget -> index list 
(* tk invocation: <widget> curselection *)

val delete : widget -> index -> index -> unit 
(* tk invocation: <widget> delete <index> <index> *)

val get : widget -> index -> string 
(* tk invocation: <widget> get <index> *)

val get_range : widget -> index -> index -> string list 
(* tk invocation: <widget> get <index> <index> *)

val index : widget -> index -> int 
(* tk invocation: <widget> index <index> *)

val insert : widget -> index -> string list -> unit 
(* tk invocation: <widget> insert <index> <string list> *)

val nearest : widget -> int -> index 
(* tk invocation: <widget> nearest <int> *)

val scan_dragto : widget -> int -> int -> unit 
(* tk invocation: <widget> scan dragto <int> <int> *)

val scan_mark : widget -> int -> int -> unit 
(* tk invocation: <widget> scan mark <int> <int> *)

val see : widget -> index -> unit 
(* tk invocation: <widget> see <index> *)

val selection_anchor : widget -> index -> unit 
(* tk invocation: <widget> selection anchor <index> *)

val selection_clear : widget -> index -> index -> unit 
(* tk invocation: <widget> selection clear <index> <index> *)

val selection_includes : widget -> index -> bool 
(* tk invocation: <widget> selection includes <index> *)

val selection_set : widget -> index -> index -> unit 
(* tk invocation: <widget> selection set <index> <index> *)

val size : widget -> int 
(* tk invocation: <widget> size *)

val xview : widget -> scrollValue -> unit 
(* tk invocation: <widget> xview <scrollValue> *)

val xview_get : widget -> float * float 
(* tk invocation: <widget> xview *)

val xview_index : widget -> index -> unit 
(* tk invocation: <widget> xview <index> *)

val yview : widget -> scrollValue -> unit 
(* tk invocation: <widget> yview <scrollValue> *)

val yview_get : widget -> float * float 
(* tk invocation: <widget> yview *)

val yview_index : widget -> index -> unit 
(* tk invocation: <widget> yview <index> *)

end

module Grab : sig
(* The grab commands  *)
open Tk
open Widget
open Textvariable


val release : widget -> unit 
(* tk invocation: grab release <widget> *)

val set : widget -> unit 
(* tk invocation: grab <widget> *)

val set_global : widget -> unit 
(* tk invocation: grab -global <widget> *)

val status : widget -> grabStatus 
(* tk invocation: grab status <widget> *)

end

module Button : sig
(* The button widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val flash : widget -> unit 
(* tk invocation: <widget> flash *)

val invoke : widget -> unit 
(* tk invocation: <widget> invoke *)

end

module Grid : sig
(* The grid commands  *)
open Tk
open Widget
open Textvariable
val all_slaves : widget -> widget list 
(* tk invocation: grid slaves <widget> *)

val bbox : widget -> int -> int -> int * int * int * int 
(* tk invocation: grid bbox <widget> <int> <int> *)

val column_configure : widget -> int -> gridopt list -> unit 
(* tk invocation: grid columnconfigure <widget> <int> <gridopt list> *)

val column_slaves : widget -> int -> widget list 
(* tk invocation: grid slaves <widget> -column <int> *)

val configure : widget list -> options list -> unit 
(* tk invocation: grid configure <widget list> <options list> *)

val forget : widget list -> unit 
(* tk invocation: grid forget <widget list> *)

val info : widget -> string 
(* tk invocation: grid info <widget> *)

val location : widget -> units -> units -> int * int 
(* tk invocation: grid location <widget> <units> <units> *)

val propagate_get : widget -> bool 
(* tk invocation: grid propagate <widget> *)

val propagate_set : widget -> bool -> unit 
(* tk invocation: grid propagate <widget> <bool> *)

val row_configure : widget -> int -> gridopt list -> unit 
(* tk invocation: grid rowconfigure <widget> <int> <gridopt list> *)

val row_slaves : widget -> int -> widget list 
(* tk invocation: grid slaves <widget> -row <int> *)

val size : widget -> int * int 
(* tk invocation: grid size <widget> *)

end

module Imagephoto : sig
(* The imagephoto commands  *)
open Tk
open Widget
open Textvariable
val blank : imagePhoto -> unit 
(* tk invocation: <imagePhoto> blank *)

val configure : imagePhoto -> options list -> unit 
(* tk invocation: <imagePhoto> configure <options list> *)

val configure_get : imagePhoto -> string 
(* tk invocation: <imagePhoto> configure *)

val copy : imagePhoto -> imagePhoto -> photo list -> unit 
(* tk invocation: <imagePhoto> copy <imagePhoto> <photo list> *)

val create : options list -> imagePhoto 
(* tk invocation: image create photo <options list> *)

val delete : imagePhoto -> unit 
(* tk invocation: image delete <imagePhoto> *)

val get : imagePhoto -> int -> int -> int * int * int 
(* tk invocation: <imagePhoto> get <int> <int> *)

val height : imagePhoto -> int 
(* tk invocation: image height <imagePhoto> *)

val read : imagePhoto -> string -> photo list -> unit 
(* tk invocation: <imagePhoto> read <string> <photo list> *)

val redither : imagePhoto -> unit 
(* tk invocation: <imagePhoto> redither *)

val width : imagePhoto -> int 
(* tk invocation: image width <imagePhoto> *)

val write : imagePhoto -> photo list -> unit 
(* tk invocation: <imagePhoto> write <photo list> *)

end

module Bell : sig
(* The bell commands  *)
open Tk
open Widget
open Textvariable
val ring : unit ->unit 
(* tk invocation: bell *)

val ring_displayof : widget -> unit 
(* tk invocation: bell -displayof <widget> *)

end

module Winfo : sig
(* The winfo commands  *)
open Tk
open Widget
open Textvariable




val cells : widget -> int 
(* tk invocation: winfo cells <widget> *)

val children : widget -> widget list 
(* tk invocation: winfo children <widget> *)

val class_name : widget -> string 
(* tk invocation: winfo class <widget> *)

val colormapfull : widget -> bool 
(* tk invocation: winfo colormapfull <widget> *)



val depth : widget -> int 
(* tk invocation: winfo depth <widget> *)

val exists : widget -> bool 
(* tk invocation: winfo exists <widget> *)

val fpixels : widget -> units -> float 
(* tk invocation: winfo fpixels <widget> <units> *)

val geometry : widget -> string 
(* tk invocation: winfo geometry <widget> *)

val height : widget -> int 
(* tk invocation: winfo height <widget> *)




val ismapped : widget -> bool 
(* tk invocation: winfo ismapped <widget> *)

val manager : widget -> string 
(* tk invocation: winfo manager <widget> *)

val name : widget -> string 
(* tk invocation: winfo name <widget> *)




val pixels : widget -> units -> int 
(* tk invocation: winfo pixels <widget> <units> *)

val pointerx : widget -> int 
(* tk invocation: winfo pointerx <widget> *)

val pointerxy : widget -> int * int 
(* tk invocation: winfo pointerxy <widget> *)

val pointery : widget -> int 
(* tk invocation: winfo pointery <widget> *)

val reqheight : widget -> int 
(* tk invocation: winfo reqheight <widget> *)

val reqwidth : widget -> int 
(* tk invocation: winfo reqwidth <widget> *)

val rgb : widget -> color -> int * int * int 
(* tk invocation: winfo rgb <widget> <color> *)

val rootx : widget -> int 
(* tk invocation: winfo rootx <widget> *)

val rooty : widget -> int 
(* tk invocation: winfo rooty <widget> *)

val screen : widget -> string 
(* tk invocation: winfo screen <widget> *)

val screencells : widget -> int 
(* tk invocation: winfo screencells <widget> *)

val screendepth : widget -> int 
(* tk invocation: winfo screendepth <widget> *)

val screenheight : widget -> int 
(* tk invocation: winfo screenheight <widget> *)

val screenmmdepth : widget -> int 
(* tk invocation: winfo screenmmdepth <widget> *)

val screenmmheight : widget -> int 
(* tk invocation: winfo screenmmheight <widget> *)

val screenvisual : widget -> string 
(* tk invocation: winfo screenvisual <widget> *)

val screenwidth : widget -> int 
(* tk invocation: winfo screenwidth <widget> *)



val viewable : widget -> bool 
(* tk invocation: winfo viewable <widget> *)

val visual : widget -> string 
(* tk invocation: winfo visual <widget> *)

val visualsavailable : widget -> string 
(* tk invocation: winfo visualsavailable <widget> *)

val vrootheight : widget -> int 
(* tk invocation: winfo vrootheight <widget> *)

val vrootwidth : widget -> int 
(* tk invocation: winfo vrootwidth <widget> *)

val vrootx : widget -> int 
(* tk invocation: winfo vrootx <widget> *)

val vrooty : widget -> int 
(* tk invocation: winfo vrooty <widget> *)

val width : widget -> int 
(* tk invocation: winfo width <widget> *)

val x : widget -> int 
(* tk invocation: winfo x <widget> *)

val y : widget -> int 
(* tk invocation: winfo y <widget> *)

val contained : int -> int -> widget -> bool
(* [contained x y w] returns true if (x,y) is in w *)


end

module Imagebitmap : sig
(* The imagebitmap commands  *)
open Tk
open Widget
open Textvariable
val configure : imageBitmap -> options list -> unit 
(* tk invocation: <imageBitmap> configure <options list> *)

val configure_get : imageBitmap -> string 
(* tk invocation: <imageBitmap> configure *)

val create : options list -> imageBitmap 
(* tk invocation: image create bitmap <options list> *)

val create_named : imageBitmap -> options list -> imageBitmap 
(* tk invocation: image create bitmap <imageBitmap> <options list> *)

val delete : imageBitmap -> unit 
(* tk invocation: image delete <imageBitmap> *)

val height : imageBitmap -> int 
(* tk invocation: image height <imageBitmap> *)

val width : imageBitmap -> int 
(* tk invocation: image width <imageBitmap> *)

end

module Tkwait : sig
(* The tkwait commands  *)
open Tk
open Widget
open Textvariable
val variable : textVariable -> unit 
(* tk invocation: tkwait variable <textVariable> *)

val visibility : widget -> unit 
(* tk invocation: tkwait visibility <widget> *)

val window : widget -> unit 
(* tk invocation: tkwait window <widget> *)

end

module Scrollbar : sig
(* The scrollbar widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val activate : widget -> widgetElement -> unit 
(* tk invocation: <widget> activate <widgetElement> *)

val activate_get : widget -> widgetElement 
(* tk invocation: <widget> activate *)

val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val delta : widget -> int -> int -> float 
(* tk invocation: <widget> delta <int> <int> *)

val fraction : widget -> int -> int -> float 
(* tk invocation: <widget> fraction <int> <int> *)

val get : widget -> float * float 
(* tk invocation: <widget> get *)

val identify : widget -> int -> int -> widgetElement 
(* tk invocation: <widget> identify <int> <int> *)

val old_get : widget -> int * int * int * int 
(* tk invocation: <widget> get *)

val old_set : widget -> int -> int -> int -> int -> unit 
(* tk invocation: <widget> set <int> <int> <int> <int> *)

val set : widget -> float -> float -> unit 
(* tk invocation: <widget> set <float> <float> *)

end

module Checkbutton : sig
(* The checkbutton widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val deselect : widget -> unit 
(* tk invocation: <widget> deselect *)

val flash : widget -> unit 
(* tk invocation: <widget> flash *)

val invoke : widget -> unit 
(* tk invocation: <widget> invoke *)

val select : widget -> unit 
(* tk invocation: <widget> select *)

val toggle : widget -> unit 
(* tk invocation: <widget> toggle *)

end

module Palette : sig
(* The palette commands  *)
open Tk
open Widget
open Textvariable
val bisque : unit ->unit 
(* tk invocation: tk_bisque *)

val set_background : color -> unit 
(* tk invocation: tk_setPalette <color> *)

end

module Focus : sig
(* The focus commands  *)
open Tk
open Widget
open Textvariable

val follows_mouse : unit ->unit 
(* tk invocation: tk_focusFollowsMouse *)

val force : widget -> unit 
(* tk invocation: focus -force <widget> *)





val set : widget -> unit 
(* tk invocation: focus <widget> *)

end

module Optionmenu : sig
(* The optionmenu commands  *)
open Tk
open Widget
open Textvariable
(* Support for tk_optionMenu *)
val create: widget -> textVariable -> string list -> widget * widget
      	      (* [create parent var options] creates a multi-option 
      	       	 menubutton and its associated menu. The option is also stored
                 in the variable. Both widgets (menubutton and menu) are
		 returned *)


end

module Wm : sig
(* The wm commands  *)
open Tk
open Widget
open Textvariable
val aspect_get : widget -> int * int * int * int 
(* tk invocation: wm aspect <widget> *)

val aspect_set : widget -> int -> int -> int -> int -> unit 
(* tk invocation: wm aspect <widget> <int> <int> <int> <int> *)

val client_get : widget -> string 
(* tk invocation: wm client <widget> *)

val client_set : widget -> string -> unit 
(* tk invocation: wm client <widget> <string> *)


val colormapwindows_set : widget -> widget list -> unit 
(* tk invocation: wm colormapwindows <widget> {<widget list>} *)

val command_clear : widget -> unit 
(* tk invocation: wm command <widget>  *)

val command_get : widget -> string list 
(* tk invocation: wm command <widget> *)

val command_set : widget -> string list -> unit 
(* tk invocation: wm command <widget> {<string list>} *)

val deiconify : widget -> unit 
(* tk invocation: wm deiconify <widget> *)

val focusmodel_get : widget -> focusModel 
(* tk invocation: wm focusmodel <widget> *)

val focusmodel_set : widget -> focusModel -> unit 
(* tk invocation: wm focusmodel <widget> <focusModel> *)

val frame : widget -> string 
(* tk invocation: wm frame <widget> *)

val geometry_get : widget -> string 
(* tk invocation: wm geometry <widget> *)

val geometry_set : widget -> string -> unit 
(* tk invocation: wm geometry <widget> <string> *)

val grid_clear : widget -> unit 
(* tk invocation: wm grid <widget>     *)

val grid_get : widget -> int * int * int * int 
(* tk invocation: wm grid <widget> *)

val grid_set : widget -> int -> int -> int -> int -> unit 
(* tk invocation: wm grid <widget> <int> <int> <int> <int> *)

val group_clear : widget -> unit 
(* tk invocation: wm group <widget>  *)


val group_set : widget -> widget -> unit 
(* tk invocation: wm group <widget> <widget> *)

val iconbitmap_clear : widget -> unit 
(* tk invocation: wm iconbitmap <widget>  *)

val iconbitmap_get : widget -> bitmap 
(* tk invocation: wm iconbitmap <widget> *)

val iconbitmap_set : widget -> bitmap -> unit 
(* tk invocation: wm iconbitmap <widget> <bitmap> *)

val iconify : widget -> unit 
(* tk invocation: wm iconify <widget> *)

val iconmask_clear : widget -> unit 
(* tk invocation: wm iconmask <widget>  *)

val iconmask_get : widget -> bitmap 
(* tk invocation: wm iconmask <widget> *)

val iconmask_set : widget -> bitmap -> unit 
(* tk invocation: wm iconmask <widget> <bitmap> *)

val iconname_get : widget -> string 
(* tk invocation: wm iconname <widget> *)

val iconname_set : widget -> string -> unit 
(* tk invocation: wm iconname <widget> <string> *)

val iconposition_clear : widget -> unit 
(* tk invocation: wm iconposition <widget>   *)

val iconposition_get : widget -> int * int 
(* tk invocation: wm iconposition <widget> *)

val iconposition_set : widget -> int -> int -> unit 
(* tk invocation: wm iconposition <widget> <int> <int> *)

val iconwindow_clear : widget -> unit 
(* tk invocation: wm iconwindow <widget>  *)


val iconwindow_set : widget -> widget -> unit 
(* tk invocation: wm iconwindow <widget> <widget> *)

val maxsize_get : widget -> int * int 
(* tk invocation: wm maxsize <widget> *)

val maxsize_set : widget -> int -> int -> unit 
(* tk invocation: wm maxsize <widget> <int> <int> *)

val minsize_get : widget -> int * int 
(* tk invocation: wm minsize <widget> *)

val minsize_set : widget -> int -> int -> unit 
(* tk invocation: wm minsize <widget> <int> <int> *)

val overrideredirect_get : widget -> bool 
(* tk invocation: wm overrideredirect <widget> *)

val overrideredirect_set : widget -> bool -> unit 
(* tk invocation: wm overrideredirect <widget> <bool> *)

val positionfrom_clear : widget -> unit 
(* tk invocation: wm positionfrom <widget>  *)

val positionfrom_get : widget -> wmFrom 
(* tk invocation: wm positionfrom <widget> *)

val positionfrom_set : widget -> wmFrom -> unit 
(* tk invocation: wm positionfrom <widget> <wmFrom> *)

val protocol_clear : widget -> string -> unit 
(* tk invocation: wm protocol <widget> <string>  *)

val protocol_set : widget -> string -> (unit -> unit) -> unit 
(* tk invocation: wm protocol <widget> <string> <(unit -> unit)> *)

val protocols : widget -> string list 
(* tk invocation: wm protocol <widget> *)

val resizable_get : widget -> bool * bool 
(* tk invocation: wm resizable <widget> *)

val resizable_set : widget -> bool -> bool -> unit 
(* tk invocation: wm resizable <widget> <bool> <bool> *)

val sizefrom_clear : widget -> unit 
(* tk invocation: wm sizefrom <widget>  *)

val sizefrom_get : widget -> wmFrom 
(* tk invocation: wm sizefrom <widget> *)

val sizefrom_set : widget -> wmFrom -> unit 
(* tk invocation: wm sizefrom <widget> <wmFrom> *)

val state : widget -> string 
(* tk invocation: wm state <widget> *)

val title_get : widget -> string 
(* tk invocation: wm title <widget> *)

val title_set : widget -> string -> unit 
(* tk invocation: wm title <widget> <string> *)

val transient_clear : widget -> unit 
(* tk invocation: wm transient <widget>  *)


val transient_set : widget -> widget -> unit 
(* tk invocation: wm transient <widget> <widget> *)

val withdraw : widget -> unit 
(* tk invocation: wm withdraw <widget> *)

end

module Entry : sig
(* The entry widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val delete_range : widget -> index -> index -> unit 
(* tk invocation: <widget> delete <index> <index> *)

val delete_single : widget -> index -> unit 
(* tk invocation: <widget> delete <index> *)

val get : widget -> string 
(* tk invocation: <widget> get *)

val icursor : widget -> index -> unit 
(* tk invocation: <widget> icursor <index> *)

val index : widget -> index -> int 
(* tk invocation: <widget> index <index> *)

val insert : widget -> index -> string -> unit 
(* tk invocation: <widget> insert <index> <string> *)

val scan_dragto : widget -> int -> unit 
(* tk invocation: <widget> scan dragto <int> *)

val scan_mark : widget -> int -> unit 
(* tk invocation: <widget> scan mark <int> *)

val selection_adjust : widget -> index -> unit 
(* tk invocation: <widget> selection adjust <index> *)

val selection_clear : widget -> unit 
(* tk invocation: <widget> selection clear *)

val selection_from : widget -> index -> unit 
(* tk invocation: <widget> selection from <index> *)

val selection_present : widget -> bool 
(* tk invocation: <widget> selection present *)

val selection_range : widget -> index -> index -> unit 
(* tk invocation: <widget> selection range <index> <index> *)

val selection_to : widget -> index -> unit 
(* tk invocation: <widget> selection to <index> *)

val xview : widget -> scrollValue -> unit 
(* tk invocation: <widget> xview <scrollValue> *)

val xview_get : widget -> float * float 
(* tk invocation: <widget> xview *)

val xview_index : widget -> index -> unit 
(* tk invocation: <widget> xview <index> *)

end

module Text : sig
(* The text widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val bbox : widget -> textIndex -> int * int * int * int 
(* tk invocation: <widget> bbox <textIndex> *)

val compare : widget -> textIndex -> comparison -> textIndex -> bool 
(* tk invocation: <widget> compare <textIndex> <comparison> <textIndex> *)

val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val debug : widget -> bool -> unit 
(* tk invocation: <widget> debug <bool> *)

val delete : widget -> textIndex -> textIndex -> unit 
(* tk invocation: <widget> delete <textIndex> <textIndex> *)

val delete_char : widget -> textIndex -> unit 
(* tk invocation: <widget> delete <textIndex> *)

val dlineinfo : widget -> textIndex -> int * int * int * int * int 
(* tk invocation: <widget> dlineinfo <textIndex> *)

val get : widget -> textIndex -> textIndex -> string 
(* tk invocation: <widget> get <textIndex> <textIndex> *)

val get_char : widget -> textIndex -> string 
(* tk invocation: <widget> get <textIndex> *)

val index : widget -> textIndex -> index 
(* tk invocation: <widget> index <textIndex> *)

val insert : widget -> textIndex -> string -> textTag list -> unit 
(* tk invocation: <widget> insert <textIndex> <string> {<textTag list>} *)

val mark_gravity_get : widget -> textMark -> markDirection 
(* tk invocation: <widget> mark gravity <textMark> *)

val mark_gravity_set : widget -> textMark -> markDirection -> unit 
(* tk invocation: <widget> mark gravity <textMark> <markDirection> *)

val mark_names : widget -> textMark list 
(* tk invocation: <widget> mark names *)

val mark_set : widget -> textMark -> textIndex -> unit 
(* tk invocation: <widget> mark set <textMark> <textIndex> *)

val mark_unset : widget -> textMark list -> unit 
(* tk invocation: <widget> mark unset <textMark list> *)

val scan_dragto : widget -> int -> int -> unit 
(* tk invocation: <widget> scan dragto <int> <int> *)

val scan_mark : widget -> int -> int -> unit 
(* tk invocation: <widget> scan mark <int> <int> *)

val search : widget -> textSearch list -> string -> textIndex -> textIndex -> index 
(* tk invocation: <widget> search <textSearch list> -- <string> <textIndex> <textIndex> *)

val see : widget -> textIndex -> unit 
(* tk invocation: <widget> see <textIndex> *)

val tag_add : widget -> textTag -> textIndex -> textIndex -> unit 
(* tk invocation: <widget> tag add <textTag> <textIndex> <textIndex> *)

val tag_add_char : widget -> textTag -> textIndex -> unit 
(* tk invocation: <widget> tag add <textTag> <textIndex> *)

val tag_allnames : widget -> textTag list 
(* tk invocation: <widget> tag names *)

val tag_configure : widget -> textTag -> options list -> unit 
(* tk invocation: <widget> tag configure <textTag> <options list> *)

val tag_delete : widget -> textTag list -> unit 
(* tk invocation: <widget> tag delete <textTag list> *)

val tag_indexnames : widget -> textIndex -> textTag list 
(* tk invocation: <widget> tag names <textIndex> *)

val tag_lower_below : widget -> textTag -> textTag -> unit 
(* tk invocation: <widget> tag lower <textTag> <textTag> *)

val tag_lower_bot : widget -> textTag -> unit 
(* tk invocation: <widget> tag lower <textTag> *)

val tag_nextrange : widget -> textTag -> textIndex -> textIndex -> index * index 
(* tk invocation: <widget> tag nextrange <textTag> <textIndex> <textIndex> *)

val tag_raise_above : widget -> textTag -> textTag -> unit 
(* tk invocation: <widget> tag raise <textTag> <textTag> *)

val tag_raise_top : widget -> textTag -> unit 
(* tk invocation: <widget> tag raise <textTag> *)

val tag_ranges : widget -> textTag -> index list 
(* tk invocation: <widget> tag ranges <textTag> *)

val tag_remove : widget -> textTag -> textIndex -> textIndex -> unit 
(* tk invocation: <widget> tag remove <textTag> <textIndex> <textIndex> *)

val tag_remove_char : widget -> textTag -> textIndex -> unit 
(* tk invocation: <widget> tag remove <textTag> <textIndex> *)

val window_configure : widget -> textTag -> options list -> unit 
(* tk invocation: <widget> window configure <textTag> <options list> *)

val window_create : widget -> textIndex -> options list -> unit 
(* tk invocation: <widget> window create <textIndex> <options list> *)

val window_names : widget -> widget list 
(* tk invocation: <widget> window names *)

val xview : widget -> scrollValue -> unit 
(* tk invocation: <widget> xview <scrollValue> *)

val xview_get : widget -> float * float 
(* tk invocation: <widget> xview *)

val yview : widget -> scrollValue -> unit 
(* tk invocation: <widget> yview <scrollValue> *)

val yview_get : widget -> float * float 
(* tk invocation: <widget> yview *)

val yview_index : widget -> textIndex -> unit 
(* tk invocation: <widget> yview <textIndex> *)

val yview_index_pickplace : widget -> textIndex -> unit 
(* tk invocation: <widget> yview -pickplace <textIndex> *)

val yview_line : widget -> int -> unit 
(* tk invocation: <widget> yview <int> *)

val tag_bind: widget -> textTag ->
                       (modifier list * xEvent) list -> bindAction -> unit 


end

module Pixmap : sig
(* The pixmap commands  *)
open Tk
open Widget
open Textvariable
open Tk
(*
 * Minimal pixmap support
 *)

type t
type pixel

val width : t -> int
    (* [width pixmap] *)
val height : t -> int
    (* [height pixmap] *)

val create : int -> int -> t
    (* [create width height] *)
val get : imagePhoto -> t
    (* [get img] *)
val set : imagePhoto -> t -> unit
    (* [set img pixmap] *)
val blit : imagePhoto -> t -> int -> int -> int -> int -> unit
    (* [blit img pixmap x y w h] (all ints must be non-negative) *)
val from_file : string -> t
    (* [from_file filename] *)

val copy : t -> t -> unit
    (* [copy src dst] *)

(*
 * Pixel operations
 *)
val get_pixel : t -> int -> int -> pixel
    (* [get_pixel pixmap x y] *)
val set_pixel : t -> int -> int -> pixel -> unit
    (* [set_pixel pixmap x y pixel] *)
val default_color : pixel

val pixel : int -> int -> int -> pixel
    (* [pixel r g b]   (r,g,b must be in [0..255]) *)

(*-*)


end

module Resource : sig
(* The resource commands  *)
open Tk
open Widget
open Textvariable

val clear : unit ->unit 
(* tk invocation: option clear *)

val get : widget -> string -> string -> string 
(* tk invocation: option get <widget> <string> <string> *)


end

module Message : sig
(* The message widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

end

module Clipboard : sig
(* The clipboard commands  *)
open Tk
open Widget
open Textvariable
val append : icccm list -> string -> unit 
(* tk invocation: clipboard append <icccm list> -- <string> *)

val clear : icccm list -> unit 
(* tk invocation: clipboard clear <icccm list> *)

end

module Menubutton : sig
(* The menubutton widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

end

module Label : sig
(* The label widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

end

module Place : sig
(* The place commands  *)
open Tk
open Widget
open Textvariable
val configure : widget -> options list -> unit 
(* tk invocation: place configure <widget> <options list> *)

val forget : widget -> unit 
(* tk invocation: place forget <widget> *)

val info : widget -> string 
(* tk invocation: place info <widget> *)

val slaves : widget -> widget list 
(* tk invocation: place slaves <widget> *)

end

module Menu : sig
(* The menu widget *)
open Tk
open Widget
open Textvariable
val create : widget -> options list -> widget 
             (* [create p options] creates a new widget with parent p.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val create_named : widget -> string -> options list -> widget 
             (* [create p name options] creates a new widget with
                parent p and new patch component name.
                Options are restricted to the widget class subset,
                and checked dynamically. *)
val activate : widget -> index -> unit 
(* tk invocation: <widget> activate <index> *)

val add_cascade : widget -> options list -> unit 
(* tk invocation: <widget> add cascade <options list> *)

val add_checkbutton : widget -> options list -> unit 
(* tk invocation: <widget> add checkbutton <options list> *)

val add_command : widget -> options list -> unit 
(* tk invocation: <widget> add command <options list> *)

val add_radiobutton : widget -> options list -> unit 
(* tk invocation: <widget> add radiobutton <options list> *)

val add_separator : widget -> unit 
(* tk invocation: <widget> add separator *)

val configure : widget -> options list -> unit 
(* tk invocation: <widget> configure <options list> *)

val configure_cascade : widget -> index -> options list -> unit 
(* tk invocation: <widget> entryconfigure <index> <options list> *)

val configure_checkbutton : widget -> index -> options list -> unit 
(* tk invocation: <widget> entryconfigure <index> <options list> *)

val configure_command : widget -> index -> options list -> unit 
(* tk invocation: <widget> entryconfigure <index> <options list> *)

val configure_get : widget -> string 
(* tk invocation: <widget> configure *)

val configure_radiobutton : widget -> index -> options list -> unit 
(* tk invocation: <widget> entryconfigure <index> <options list> *)

val delete : widget -> index -> index -> unit 
(* tk invocation: <widget> delete <index> <index> *)

val entryconfigure_get : widget -> index -> string 
(* tk invocation: <widget> entryconfigure <index> *)

val index : widget -> index -> int 
(* tk invocation: <widget> index <index> *)

val insert_cascade : widget -> index -> options list -> unit 
(* tk invocation: <widget> insert <index> cascade <options list> *)

val insert_checkbutton : widget -> index -> options list -> unit 
(* tk invocation: <widget> insert <index> checkbutton <options list> *)

val insert_command : widget -> index -> options list -> unit 
(* tk invocation: <widget> insert <index> command <options list> *)

val insert_radiobutton : widget -> index -> options list -> unit 
(* tk invocation: <widget> insert <index> radiobutton <options list> *)

val insert_separator : widget -> index -> unit 
(* tk invocation: <widget> insert <index> separator *)

val invoke : widget -> index -> string 
(* tk invocation: <widget> invoke <index> *)

val popup : widget -> int -> int -> unit 
(* tk invocation: tk_popup <widget> <int> <int> *)

val popup_entry : widget -> int -> int -> index -> unit 
(* tk invocation: tk_popup <widget> <int> <int> <index> *)

val post : widget -> int -> int -> unit 
(* tk invocation: <widget> post <int> <int> *)

val postcascade : widget -> index -> unit 
(* tk invocation: <widget> postcascade <index> *)

val typeof : widget -> index -> menuItem 
(* tk invocation: <widget> type <index> *)

val unpost : widget -> unit 
(* tk invocation: <widget> unpost *)

val yposition : widget -> index -> int 
(* tk invocation: <widget> yposition <index> *)

end

